@@ -18,12 +18,13 @@ def create_api_error_from_http_exception(e):
     try:
         explanation = response.json()['message']
     except ValueError:
-        explanation = response.content.strip()
+        explanation = (response.content or '').strip()
     cls = APIError
     if response.status_code == 404:
         if explanation and ('No such image' in str(explanation) or
                             'not found: does not exist or no pull access'
-                            in str(explanation)):
+                            in str(explanation) or
+                            'repository does not exist' in str(explanation)):
             cls = ImageNotFound
         else:
             cls = NotFound
@@ -59,7 +60,7 @@ class APIError(requests.exceptions.HTTPError, DockerException):
 
     @property
     def status_code(self):
-        if self.response:
+        if self.response is not None:
             return self.response.status_code
 
     def is_client_error(self):
@@ -126,8 +127,11 @@ class ContainerError(DockerException):
         self.command = command
         self.image = image
         self.stderr = stderr
-        msg = ("Command '{}' in image '{}' returned non-zero exit status {}: "
-               "{}").format(command, image, exit_status, stderr)
+
+        err = ": {}".format(stderr) if stderr is not None else ""
+        msg = ("Command '{}' in image '{}' returned non-zero exit "
+               "status {}{}").format(command, image, exit_status, err)
+
         super(ContainerError, self).__init__(msg)
 
 
