@@ -153,6 +153,10 @@ class ImageCollection(Collection):
                 Dockerfile
             network_mode (str): networking mode for the run commands during
                 build
+            squash (bool): Squash the resulting images layers into a
+                single layer.
+            extra_hosts (dict): Extra hosts to add to /etc/hosts in building
+                containers, as a mapping of hostname to IP address.
 
         Returns:
             (:py:class:`Image`): The built image.
@@ -224,7 +228,7 @@ class ImageCollection(Collection):
                 If the server returns an error.
         """
         resp = self.client.api.images(name=name, all=all, filters=filters)
-        return [self.prepare_model(r) for r in resp]
+        return [self.get(r["Id"]) for r in resp]
 
     def load(self, data):
         """
@@ -235,6 +239,9 @@ class ImageCollection(Collection):
         Args:
             data (binary): Image data to be loaded.
 
+        Returns:
+            (generator): Progress output as JSON objects
+
         Raises:
             :py:class:`docker.errors.APIError`
                 If the server returns an error.
@@ -251,7 +258,7 @@ class ImageCollection(Collection):
         low-level API.
 
         Args:
-            repository (str): The repository to pull
+            name (str): The repository to pull
             tag (str): The tag to pull
             insecure_registry (bool): Use an insecure registry
             auth_config (dict): Override the credentials that
